@@ -57,11 +57,12 @@ else:
 _LOGGER.addHandler(_syslog_handler)
 
 # Create a handler to log messages to stderr
-_stderr_formatter = logging.Formatter('\n\n******** LOG NOTE ********\n%(message)s\n')
+_stderr_formatter = logging.Formatter('\n\n**** LOG NOTE ****\n%(message)s\n')
 _stderr_handler = logging.StreamHandler()
 _stderr_handler.setFormatter(_stderr_formatter)
 _LOGGER.addHandler(_stderr_handler)
 
+
 def import_module(name):
     """ Imports a module into the current runtime environment
 
@@ -101,6 +102,7 @@ def import_module(name):
 
     return mod
 
+
 def load_module(name):
     """ Attempts to load a module into the current environment
 
@@ -127,6 +129,7 @@ def load_module(name):
             raise ImportError('unable to import module %s' % name)
         return mod
 
+
 class ProxyCall(object):
 
     def __init__(self, proxy, method):
@@ -136,6 +139,7 @@ class ProxyCall(object):
     def __call__(self, *args, **kwargs):
         return self.proxy(self.method, *args, **kwargs)
 
+
 def islocalconnection():
     """ Checks if running locally on EOS device or remotely
 
@@ -149,6 +153,7 @@ def islocalconnection():
     """
     return os.path.exists('/etc/Eos-release')
 
+
 def debug(text):
     """Log a message to syslog and stderr
 
@@ -162,6 +167,7 @@ def debug(text):
     msg = "%s.%s: %s" % (module, func, text)
     _LOGGER.debug(msg)
 
+
 def make_iterable(value):
     """Converts the supplied value to a list object
 
@@ -174,6 +180,10 @@ def make_iterable(value):
     Returns:
         An iterable object of type list
     """
+    if sys.version_info <= (3, 0):
+        # Convert unicode values to strings for Python 2
+        if isinstance(value, unicode):
+            value = str(value)
     if isinstance(value, str):
         value = [value]
 
@@ -182,11 +192,13 @@ def make_iterable(value):
 
     return value
 
+
 def lookahead(it):
     it1, it2 = tee(iter(it))
     next(it2)
     return zip_longest(it1, it2)
 
+
 def expand_range(arg, value_delimiter=',', range_delimiter='-'):
     """
     Expands a delimited string of ranged integers into a list of strings
@@ -209,6 +221,7 @@ def expand_range(arg, value_delimiter=',', range_delimiter='-'):
             values.extend([item])
     return [str(x) for x in values]
 
+
 def collapse_range(arg, value_delimiter=',', range_delimiter='-'):
     """
     Collapses a list of values into a range set
