@@ -51,6 +51,35 @@ class TestNode(unittest.TestCase):
         self.connection = Mock()
         self.node = pyeapi.client.Node(self.connection)
 
+    def test_get_version_properties_match_version_number_no_match_model(self):
+        self.node.enable = Mock()
+        version = '4.17.1.1F-3512479.41711F (engineering build)'
+        self.node.enable.return_value = [{'result': {'version': version,
+                                                     'modelName': 'vEOS'}}]
+        self.node._get_version_properties()
+        self.assertEqual(self.node.version_number, '4.17.1.1')
+        self.assertEqual(self.node.model, 'vEOS')
+
+    def test_get_version_properties_no_match_version_number_match_model(self):
+        self.node.enable = Mock()
+        version = 'special-4.17.1.1F-3512479.41711F (engineering build)'
+        model = 'DCS-7260QX-64-F'
+        self.node.enable.return_value = [{'result': {'version': version,
+                                                     'modelName': model}}]
+        self.node._get_version_properties()
+        self.assertEqual(self.node.version_number, version)
+        self.assertEqual(self.node.model, '7260')
+
+    def test_version_properties_populate(self):
+        self.node.enable = Mock()
+        version = '4.17.1.1F-3512479.41711F (engineering build)'
+        self.node.enable.return_value = [{'result': {'version': version,
+                                                     'modelName': 'vEOS'}}]
+        self.node.version_number
+        self.assertEqual(self.node.version_number, '4.17.1.1')
+        self.assertEqual(self.node.version, version)
+        self.assertEqual(self.node.model, 'vEOS')
+
     def test_enable_with_single_command(self):
         command = random_string()
         response = ['enable', command]
@@ -61,6 +90,17 @@ class TestNode(unittest.TestCase):
         self.connection.execute.assert_called_once_with(response, 'json')
         self.assertEqual(command, result[0]['result'])
 
+    def test_enable_with_single_unicode_command(self):
+        command = random_string()
+        command = u'%s' % command
+        response = ['enable', command]
+
+        self.connection.execute.return_value = {'result': list(response)}
+        result = self.node.enable(command)
+
+        self.connection.execute.assert_called_once_with(response, 'json')
+        self.assertEqual(command, result[0]['result'])
+
     def test_no_enable_with_single_command(self):
         command = random_string()
         response = [command]
@@ -71,8 +111,6 @@ class TestNode(unittest.TestCase):
         self.connection.execute.assert_called_once_with(response, 'json')
         self.assertEqual(command, result[0]['result'])
 
-
-
     def test_enable_with_multiple_commands(self):
         commands = list()
         for i in range(0, random_int(2, 5)):
@@ -93,6 +131,26 @@ class TestNode(unittest.TestCase):
         for index, response in enumerate(responses):
             self.assertEqual(commands[index], response['result'])
 
+    def test_enable_with_multiple_unicode_commands(self):
+        commands = list()
+        for i in range(0, random_int(2, 5)):
+            commands.append(u'%s' % random_string())
+
+        def execute_response(cmds, *args):
+            return {'result': [x for x in cmds]}
+
+        self.connection.execute.side_effect = execute_response
+
+        responses = self.node.enable(commands)
+
+        self.assertEqual(self.connection.execute.call_count, len(commands))
+
+        expected_calls = [call(['enable', cmd], 'json') for cmd in commands]
+        self.assertEqual(self.connection.execute.mock_calls, expected_calls)
+
+        for index, response in enumerate(responses):
+            self.assertEqual(commands[index], response['result'])
+
     def test_config_with_single_command(self):
         command = random_string()
         self.node.run_commands = Mock(return_value=[{}, {}])
@@ -116,7 +174,7 @@ class TestNode(unittest.TestCase):
     def test_config_with_multiple_multilines(self):
         commands = [random_string(),
                     ('banner login MULTILINE:This is a new banner\n'
-                    'with different lines!!!'),
+                     'with different lines!!!'),
                     random_string()]
 
         self.node.run_commands = Mock(return_value=[{}, {}, {}, {}])
@@ -284,7 +342,7 @@ class TestClient(unittest.TestCase):
         node.get_config = get_config_mock
         self.assertIsInstance(node.startup_config, str)
 
-    def test_node_returns_cached_startup_confgi(self):
+    def test_node_returns_cached_startup_config(self):
         node = pyeapi.client.Node(None)
         config_file = open(get_fixture('running_config.text'))
         config = config_file.read()
@@ -292,6 +350,49 @@ class TestClient(unittest.TestCase):
         node._startup_config = config
         self.assertEqual(node.startup_config, config)
 
+    def test_node_returns_version(self):
+        node = pyeapi.client.Node(None)
+        version = '4.17.1.1F-3512479.41711F (engineering build)'
+        node.enable = Mock()
+        node.enable.return_value = [{'result': {'version': version,
+                                                'modelName': 'vEOS'}}]
+        self.assertIsInstance(node.version, str)
+        self.assertEqual(node.version, version)
+
+    def test_node_returns_cached_version(self):
+        node = pyeapi.client.Node(None)
+        node._version = '4.16.7R'
+        self.assertEqual(node.version, '4.16.7R')
+
+    def test_node_returns_version_number(self):
+        node = pyeapi.client.Node(None)
+        version = '4.17.1.1F-3512479.41711F (engineering build)'
+        node.enable = Mock()
+        node.enable.return_value = [{'result': {'version': version,
+                                                'modelName': 'vEOS'}}]
+        self.assertIsInstance(node.version_number, str)
+        self.assertIn(node.version_number, version)
+
+    def test_node_returns_cached_version_number(self):
+        node = pyeapi.client.Node(None)
+        node._version_number = '4.16.7'
+        self.assertEqual(node.version_number, '4.16.7')
+
+    def test_node_returns_model(self):
+        node = pyeapi.client.Node(None)
+        version = '4.17.1.1F-3512479.41711F (engineering build)'
+        model = 'DCS-7260QX-64-F'
+        node.enable = Mock()
+        node.enable.return_value = [{'result': {'version': version,
+                                                'modelName': model}}]
+        self.assertIsInstance(node.model, str)
+        self.assertIn(node.model, model)
+
+    def test_node_returns_cached_model(self):
+        node = pyeapi.client.Node(None)
+        node._model = '7777'
+        self.assertEqual(node.model, '7777')
+
     def test_connect_default_type(self):
         transport = Mock()
         with patch.dict(pyeapi.client.TRANSPORTS, {'https': transport}):
