@@ -23,10 +23,17 @@ class TestUtils(unittest.TestCase):
     def test_make_iterable_from_string(self):
         result = pyeapi.utils.make_iterable('test')
         self.assertIsInstance(result, collections.Iterable)
+        self.assertEquals(len(result), 1)
+
+    def test_make_iterable_from_unicode(self):
+        result = pyeapi.utils.make_iterable(u'test')
+        self.assertIsInstance(result, collections.Iterable)
+        self.assertEquals(len(result), 1)
 
     def test_make_iterable_from_iterable(self):
         result = pyeapi.utils.make_iterable(['test'])
         self.assertIsInstance(result, collections.Iterable)
+        self.assertEquals(len(result), 1)
 
     def test_make_iterable_raises_type_error(self):
         with self.assertRaises(TypeError):
